{ TAMBounceImage version 1.1 copyright  november 1997 by Alexander Meeder (ameeder@dds.nl)

  This component is usefull for simple animation, for more information see the list of special
  properties below...

  Extra properties:
  * Align same as with other Delphi components
  * AutoCapture: when true, the component wil capture the screen (desktop) when started
  * AutoHideCursor: when true, the component hides the cursor
  * Bitmap is used for the background bitmap
  * Bounce: boolean is a method to let the picture bounce on the Parent-control (Form, panel etc.)
  * BounceBitmap: a special object-property with several subproperties
  * BounceSound: the path to a *.wav-file which will be played when the BounceBitmap hit a border
  * Caption: this caption will be displayed in the center of the component
  * CenterBitmap, boolean center background true/false
  * Color used to fill the background when bitmap doesn't fill entire clientarea
  * Direction: up_right, down_right, down_left, up_left; start moving in this direction, after that is
          gives the status in which direction the picture is moving at the moment
  * Font: the font to display the caption
  * Interval: milisecond, interval between every move
  * MoveBy: between 1 and 100 pixels to move the picture
  * ParentColor to use automatically the color of the parent f.e. the Form or a panel

  Events:
  * OnBounced: every time the picture hits the border of its parent (Form, panel etc.) you can trigger
          this event and one of its parameters is the new direction it moves
  * OnClick: see Delphi help
  * OnDblClick: idem
  * OnMouseDown: idem
  * OnMouseMove: idem
  * OnMouseUp: idem
  * OnMoved: every time the picture is moved you can trigger this event

  This version is for Delphi 1.x, Delphi 2.x and Delphi 3.x
}

unit AMBncBmp;

interface

uses
  {$IFDEF WIN32}
    Windows,
  {$ELSE}
    WinTypes, WinProcs,
  {$ENDIF}
    Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls,
    MMSystem, DsgnIntf;

type
  TDirections = (up_right, down_right, down_left, up_left);
  TMoveBy = 1 .. 100;

  TBounced = procedure (Sender: TObject; Direction: TDirections) of object;

  TBounceSoundProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TAMBounceImage = class;

  TBounceBitmap = class(TPersistent)
  private
    FBitmap: TBitmap;
    FLeft: integer;
    FTop: integer;
    FTransparentColor: TColor;

    Parent: TAMBounceImage;

    procedure SetBitmap(value: TBitmap);
    procedure SetTransparentColor(value: TColor);
  protected
  public
    constructor Create;
    destructor Destroy; override;
  published
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property Left: integer read FLeft write FLeft default 0;
    property Top: integer read FTop write FTop default 0;
    property TransparentColor: TColor read FTransparentColor write SetTransparentColor default clOlive;
  end;

  TAMBounceImage = class(TGraphicControl)
  private
    FAutoHideCursor: boolean;
    FAutoScrCapt: boolean;
    FBitmap: TBitmap;
    FBounce: boolean;
    FBounceBitmap: TBounceBitmap;
    FBounceSound: TFileName;
    FBuffer: TBitmap;
    FCenter: boolean;
    FDirection: TDirections;
    FInterval: integer;
    FMoveBy: TMoveBy;
    FTimer: TTimer;

    FBounced: TBounced;
    FMoved: TNotifyEvent;

    procedure SetBitmap(value: TBitmap);
    procedure SetBounce(value: boolean);
    procedure SetBounceBitmap(value: TBounceBitmap);
    procedure SetBounceSound(value: TFileName);
    procedure SetDirection(value: TDirections);
    procedure SetInterval(value: integer);
    procedure SetMoveBy(value: TMoveBy);
  protected
    procedure CmFontChanged(var Message: TWmNoParams); message CM_FONTCHANGED;
    procedure CmTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure TimerTimer(Sender: TObject);
    procedure Loaded; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property AutoHideCursor: boolean read FAutoHideCursor write FAutoHideCursor default False;
    property AutoScrCapt: boolean read FAutoScrCapt write FAutoScrCapt default False;
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property Bounce: boolean read FBounce write SetBounce default True;
    property BounceBitmap: TBounceBitmap read FBounceBitmap write SetBounceBitmap;
    property BounceSound: TFileName read FBounceSound write SetBounceSound;
    property CenterBitmap: boolean read FCenter write FCenter default True;
    property Caption;
    property Color;
    property Direction: TDirections read FDirection write SetDirection default up_right;
    property Interval: integer read FInterval write SetInterval default 100;
    property Font;
    property MoveBy: TMoveBy read FMoveBy write SetMoveBy default 1;
    property ParentColor;
    property ShowHint;
    property OnBounced: TBounced read FBounced write FBounced;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMoved: TNotifyEvent read FMoved write FMoved;
  end;

procedure Register;

implementation

procedure TBounceSoundProperty.Edit;
var
  dlgOpen: TOpenDialog;
begin
  dlgOpen := TOpenDialog.Create(nil);
  dlgOpen.Filter := 'Wave-files|*.wav';
  dlgOpen.FileName := GetValue;
  if dlgOpen.Execute then SetValue(dlgOpen.FileName);
end;

function TBounceSoundProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;

procedure TBounceBitmap.SetBitmap(value: TBitmap);
begin
  if value <> FBitmap then
  begin
    FBitmap.Assign(value);
    if value <> nil then
    begin
      FTransparentColor := FBitmap.Canvas.Pixels[0, FBitmap.Height - 1];
      Parent.Refresh;
    end;
  end;
end;

procedure TBounceBitmap.SetTransparentColor(value: TColor);
begin
  if not (FBitmap.Empty) and (value <> FTransparentColor) then
  begin
    FTransparentColor := value;
    Parent.Refresh;
  end;
end;

constructor TBounceBitmap.Create;
begin
  inherited Create;
  FBitmap := TBitmap.Create;
  FTransparentColor := clOlive;
  FLeft := 0;
  FTop := 0;
end;

destructor TBounceBitmap.Destroy;
begin
  FBitmap.Free;
  inherited Destroy;
end;

procedure TAMBounceImage.CmFontChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TAMBounceImage.CmTextChanged(var Message: TWmNoParams);
begin
  inherited;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle - [csOpaque];
  Repaint;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
end;

procedure TAMBounceImage.SetBitmap(value: TBitmap);
begin
  FBitmap.Assign(value);
  if not FBitmap.Empty and not CenterBitmap then
  begin
    Width := FBitmap.Width;
    Height := Fbitmap.Height;
  end;
end;

procedure TAMBounceImage.SetBounce(value: boolean);
begin
  if value <> FBounce then
  begin
    FBounce := value;
    if not (csDesigning in ComponentState) then FTimer.Enabled := value;
  end;
end;

procedure TAMBounceImage.SetBounceBitmap(value: TBounceBitmap);
begin
  FBounceBitmap.Assign(value);
end;

procedure TAMBounceImage.SetBounceSound(value: TFileName);
begin
  if value <> FBounceSound then FBounceSound := value;
end;

procedure TAMBounceImage.SetDirection(value: TDirections);
begin
  if value <> FDirection then FDirection := value;
end;

procedure TAMBounceImage.SetInterval(value: integer);
begin
  if value <> FInterval then
  begin
    FInterval := value;
    FTimer.Interval := value;
  end;
end;

procedure TAMBounceImage.SetMoveBy(value: TMoveBy);
begin
  if value <> FMoveBy then FMoveBy := value;
end;

procedure TAMBounceImage.Loaded;
var
  DesktopDC: hDC;
  DesktopCanvas: TCanvas;
  DesktopRect: TRect;
begin
  if AutoScrCapt and not (csDesigning in ComponentState) then
  begin
    ShowWindow(Application.Handle, SW_HIDE);        {Hide application window}
    ShowWindow((Owner as TForm).Handle, SW_HIDE);   {Hide button from taskbar}
    DesktopDC := GetWindowDC(GetDesktopWindow);
    DesktopCanvas := TCanvas.Create;
    try
      DesktopCanvas.Handle := DesktopDC;
      DeskTopRect := Rect(0, 0, Screen.Width, Screen.Height);
      FBitmap.Width := DeskTopRect.Right - DeskTopRect.Left;
      FBitmap.Height := DeskTopRect.Bottom - DeskTopRect.Top;
      FBitmap.Canvas.CopyRect(DeskTopRect, DeskTopCanvas, DeskTopRect);
    finally
      DesktopCanvas.Free;
      ReleaseDC(GetDesktopWindow, DesktopDC);
    end;
    ShowWindow(Application.Handle, SW_SHOW);
  end;
  if AutoHideCursor then Screen.Cursor := crNone;
end;

procedure TAMBounceImage.Paint;
var
  ImageList: TImageList;
  PictureIndex: integer;
  TmpRect: TRect;
  CenterRect, OrgRect: TRect;
  CenterX, CenterY: integer;
begin
  if FBounceBitmap.Bitmap.Empty then Exit;
  inherited Paint;
  {$IFDEF WIN32}
     ImageList := TImageList.CreateSize(BounceBitmap.Bitmap.Width, BounceBitmap.Bitmap.Height);
  {$ELSE}
     ImageList := TImageList.Create(BounceBitmap.Bitmap.Width, BounceBitmap.Bitmap.Height);
  {$ENDIF}
  try
    {$IFDEF WIN32}
       ImageList.DrawingStyle := dsTransparent;
    {$ENDIF}
    TmpRect := Rect(0,0,Width,Height);
    with FBuffer do
    begin
      Width := Self.Width;
      Height := Self.Height;
      Canvas.Font := Self.Font;
      Canvas.Brush.Color := Self.Color;
      Canvas.FillRect(TmpRect);
      if not Bitmap.Empty then
      begin
        if CenterBitmap then
        begin
          CenterX := (Self.Width - Bitmap.Width) div 2;
          CenterY := (Self.Height - Bitmap.Height) div 2;
          CenterRect := Rect(CenterX, CenterY, CenterX + Bitmap.Width, CenterY + Bitmap.Height);
          OrgRect := Rect(0,0,Bitmap.Width,Bitmap.Height);
          Canvas.CopyRect(CenterRect,Bitmap.Canvas,OrgRect);
        end
        else
          Canvas.CopyRect(TmpRect,Bitmap.Canvas,TmpRect);
      end;
    end;
    PictureIndex := ImageList.AddMasked(BounceBitmap.Bitmap, BounceBitmap.TransparentColor);
    ImageList.Draw(FBuffer.Canvas, BounceBitmap.Left, BounceBitmap.Top, PictureIndex);
    CenterX := (FBuffer.Width - FBuffer.Canvas.TextWidth(Caption)) div 2;
    CenterY := (FBuffer.Height - FBuffer.Canvas.TextHeight(Caption)) div 2;
    FBuffer.Canvas.Brush.Style := bsClear;
    FBuffer.Canvas.TextOut(CenterX, CenterY, Caption);
    Self.Canvas.CopyRect(TmpRect, FBuffer.Canvas, TmpRect);
  finally
    ImageList.Free;
  end;
end;

procedure TAMBounceImage.TimerTimer(Sender: TObject);
var
  Bounced: boolean;
  WavFile: Array[0..79] of char;  
begin
  Bounced := False;
  with FBounceBitmap do
  begin
    case Direction of
      up_right   : begin
                     Left := Left + MoveBy;
                     Top := Top - MoveBy;
                     if Top <= 0 then
                     begin
                       Direction := down_right;
                       Bounced := True;
                     end;
                     if Left + FBounceBitmap.Bitmap.Width >= Self.ClientWidth then
                     begin
                       Direction := up_left;
                       Bounced := True;
                     end;
                   end;
      down_right : begin
                     Left := Left + MoveBy;
                     Top := Top + MoveBy;
                     if Top + FBounceBitmap.Bitmap.Height >= Self.ClientHeight then
                     begin
                       Direction := up_right;
                       Bounced := True;
                     end;
                     if Left + FBounceBitmap.Bitmap.Width >= Self.ClientWidth then
                     begin
                       Direction := down_left;
                       Bounced := True;
                     end;
                   end;
      down_left  : begin
                     Left := Left - MoveBy;
                     Top := Top + MoveBy;
                     if Top + FBounceBitmap.Bitmap.Height >= Self.ClientHeight then
                     begin
                       Direction := up_left;
                       Bounced := True;
                     end;
                     if Left <= 0 then
                     begin
                       Direction := down_right;
                       Bounced := True;
                     end;
                   end;
      up_left    : begin
                     Left := Left - MoveBy;
                     Top := Top - MoveBy;
                     if Top <= 0 then
                     begin
                       Direction := down_left;
                       Bounced := True;
                     end;
                     if Left <= 0 then
                     begin
                       Direction := up_right;
                       Bounced := True;
                     end;
                   end;
    end;
  end;
  Refresh;
  if Bounced then
  begin
    if BounceSound <> '' then
    begin
      StrPCopy(WavFile, BounceSound);
      sndPlaySound(WavFile, SND_ASYNC);
    end;
    if Assigned(FBounced) then FBounced(self, Direction);
  end;
  if Assigned(FMoved) then FMoved(self);
end;

constructor TAMBounceImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  SetBounds(0,0,100,100);
  FTimer := TTimer.Create(Self);
  FTimer.OnTimer := TimerTimer;
  FTimer.Enabled := False;
  FBitmap := TBitmap.Create;
  FBuffer := TBitmap.Create;
  FBounceBitmap := TBounceBitmap.Create;
  FBounceBitmap.Parent := Self;
  Bounce := True;
  FAutoHideCursor := False;
  FAutoScrCapt := False;
  FCenter := True;
  FDirection := up_right;
  FInterval := 100;
  FMoveBy := 1;
end;

destructor TAMBounceImage.Destroy;
begin
  if AutoHideCursor then Screen.Cursor := crDefault; 
  FTimer.Free;
  FBitmap.Free;
  FBuffer.Free;
  FBounceBitmap.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('AM Software', [TAMBounceImage]);
  RegisterPropertyEditor(TypeInfo(TFileName), TAMBounceImage, 'BounceSound', TBounceSoundProperty);
end;

end.
